/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set et sw=2 ts=2: */
/***************************************************************************
 *            fileoperations.cc
 *
 *  Thu Aug 26 08:46:21 CEST 2010
 *  Copyright 2010 Bent Bisballe Nyeng
 *  deva@aasimon.org
 ****************************************************************************/

/*
 *  This file is part of PC-Show.
 *
 *  PC-Show is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  PC-Show is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with PC-Show; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */
#include "fileoperations.h"

#include <QFile>
#include <QMessageBox>
#include <QSettings>
#include <QFileDialog>

#include <QDialog>
#include <QPushButton>
#include <QLineEdit>
#include <QLabel>
#include <QVBoxLayout>
#include <QHBoxLayout>

#include "renamebox.h"

void deleteSelected(ViewerItem *item)
{
  QMessageBox::StandardButton ret =
    QMessageBox::warning(NULL, QObject::tr("Delete file"),
                         QObject::tr("Are you sure you want to delete ") +
                         item->image()->filename() + QObject::tr("?"),
                         QMessageBox::Yes | QMessageBox::No,
                         QMessageBox::No);

  if(ret == QMessageBox::No) return;
  item->image()->remove();
}

void copySelected(ViewerItem *item)
{
  QSettings settings("Aasimon.org", "PC-Show");
  settings.beginGroup("Options");
  QString dst = settings.value("storepath",
                               QObject::tr("c:\\drafts")).toString();
  settings.endGroup();

  dst = QFileDialog::getExistingDirectory(NULL,
                                          QObject::tr("Choose destination"),
                                          dst);
  if(dst == "") return;

  item->image()->copy(dst);
}

void moveSelected(ViewerItem *item)
{
  QSettings settings("Aasimon.org", "PC-Show");
  settings.beginGroup("Options");
  QString dst = settings.value("storepath",
                               QObject::tr("c:\\drafts")).toString();
  settings.endGroup();

  dst = QFileDialog::getExistingDirectory(NULL,
                                          QObject::tr("Choose destination"),
                                          dst);
  if(dst == "") return;

  item->image()->move(dst);
}

void saveSelectedAs(ViewerItem *item)
{
  QSettings settings("Aasimon.org", "PC-Show");
  settings.beginGroup("Options");
  QString dst = settings.value("storepath",
                               QObject::tr("c:\\drafts")).toString();
  settings.endGroup();
  
  QString file = QFileDialog::getSaveFileName(NULL,
                                              QObject::tr("Save File"), dst);
  if(file != "") {
    item->image()->image.save(file);
  }
}

void renameSelected(ViewerItem *item)
{
  QSettings settings("Aasimon.org", "PC-Show");
  settings.beginGroup("Options");
  QString dst = settings.value("storepath",
                               QObject::tr("c:\\drafts")).toString();
  settings.endGroup();

  RenameBox renbox(QFileInfo(item->image()->filename()).fileName());
  if(renbox.exec() == QDialog::Accepted) {
    item->image()->rename(QFileInfo(renbox.filename()).fileName());
  }
}
