/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set et sw=2 ts=2: */
/***************************************************************************
 *            filewidgetitem.cc
 *
 *  Wed Oct  6 14:55:09 CEST 2010
 *  Copyright 2010 Bent Bisballe Nyeng
 *  deva@aasimon.org
 ****************************************************************************/

/*
 *  This file is part of PC-Show.
 *
 *  PC-Show is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  PC-Show is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with PC-Show; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */
#include "filewidgetitem.h"

#include <QIcon>
#include <QPixmap>
#include <QFileInfo>
#include <QDateTime>

#include <stdio.h>

FileWidgetItem::FileWidgetItem(const QString &path, const QString &file)
{
  img = new Image(path, file);
  connect(img, SIGNAL(loaded()), this, SLOT(updateImage()));
  connect(img, SIGNAL(nameChanged()), this, SLOT(updateName()));
  connect(img, SIGNAL(unloading()), this, SLOT(deleted()));

  QIcon icon(QPixmap::fromImage(img->get(64, 64)));

  setIcon(0, icon);
  updateName();
  QFileInfo info(img->filename());
  setText(2, info.created().toString(Qt::SystemLocaleShortDate));

  connect(this, SIGNAL(destroyed()),
          img, SLOT(stopLoading()));

  connect(this, SIGNAL(destroyed()),
          img, SLOT(deleteLater()), Qt::QueuedConnection);
}

FileWidgetItem::~FileWidgetItem()
{
}

void FileWidgetItem::updateImage()
{
  QIcon icon(QPixmap::fromImage(img->get(64, 64)));
  setIcon(0, icon);
}

Image *FileWidgetItem::image()
{
  return img;
}

void FileWidgetItem::deleted()
{
  QTreeWidget *t = treeWidget();
  
  int idx = t->indexOfTopLevelItem(this);
  t->takeTopLevelItem(idx);
  
  deleteLater();
}

void FileWidgetItem::updateName()
{
  QFileInfo f(img->filename());
  setText(1, f.fileName());
}
