/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set et sw=2 ts=2: */
/***************************************************************************
 *            grabber.cc
 *
 *  Mon Sep 27 13:36:07 CEST 2010
 *  Copyright 2010 Bent Bisballe Nyeng
 *  deva@aasimon.org
 ****************************************************************************/

/*
 *  This file is part of PC-Show.
 *
 *  PC-Show is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  PC-Show is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with PC-Show; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */
#include "grabber.h"

#include <QPaintEvent>
#include <QMouseEvent>
#include <QKeyEvent>
#include <QDir>
#include <QFile>
#include <QPainter>

#include <QMessageBox>

#include <math.h>
#include <stdio.h>

Grabber::Grabber(const QString &path)
{
  this->path = path;
  showFullScreen();

  current = -1;

  btn_del = btn_abo = NULL;

  QDir d(path);
  if(d.exists()) {
    watcher.addPath(path);
    connect(&watcher, SIGNAL(directoryChanged(const QString &)),
            this, SLOT(directoryChanged(const QString &)));
    directoryChanged(path);
  }
}

void Grabber::directoryChanged(const QString &path)
{
  
  QDir d(path);

  if(!d.exists()) return;

  QStringList l = d.entryList(QDir::Files);

  QStringList::iterator i;
  
  bool changed = false;
  i = l.begin();
  while(i != l.end()) {
    if(files.contains(*i) == false) {
      GrabberImage *img = new GrabberImage(this, path + "/" + *i);

      connect(img, SIGNAL(selected(GrabberImage *)),
              this, SLOT(selected(GrabberImage *)));

      connect(img, SIGNAL(rightClick()),
              this, SLOT(queryDelete()));

      images.push_back(img);
      current = images.size() - 1;
      files.append(*i);
      changed = true;
    }
    i++;
  }

  i = files.begin();
  while(i != files.end()) {
    if(l.contains(*i) == false) {
      // Remove file from screen and from filelist.
      QVector<GrabberImage*>::iterator j = images.begin();
      while(j != images.end()) {
        GrabberImage *img = *j;
        if(img->file() == path + "/" + *i) {
          delete img;
          j = images.erase(j);
          continue;
        }
        j++;
      }

      i = files.erase(i);

      changed = true;
    } else {
      i++;
    }
  }

  if(changed) repaint();
}

void Grabber::paintEvent(QPaintEvent *)
{
  QPainter p(this);

  for(int i = 0; i < images.size(); i++) {
    GrabberImage *img = images[i];

    int x;
    int y;
    int w;
    int h;
    
    if(i != current) {
      x = i * (width() / images.size());
      y = 0;
      w = width() / images.size();
      h = height() / 10;
    } else {
      x = 0;
      y = height() / 10;
      w = width();
      h = (height() / 10) * 9;

      int xl = i * (width() / images.size());
      p.drawEllipse(xl, 0, width() / images.size(), height() / 10);
      /*
      p.drawImage(xl, 0,
                  img->img.scaled(width() / images.size(),
                                  height() / 10,
                                  Qt::KeepAspectRatio));
      p.drawRect(xl, 0, width() / images.size(), height() / 10);
      */
    }

    img->resize(w, h);
    img->move(x, y);
    img->show();
  }

  p.drawText(0, 10, tr("ESC: Delete all and close."));
  p.drawText(0, 20, tr("Enter: Copy all to patient folder and close."));
}

void Grabber::mousePressEvent(QMouseEvent *)
{
}

void Grabber::selected(GrabberImage *sel)
{
  current = -1;
  for(int i = 0; i < images.size(); i++) {
    GrabberImage *img = images[i];
    if(img == sel) {
      current = i;
      repaint();
      break;
    }
  }
}

void Grabber::doDelete()
{
  if(current == -1) return;
  GrabberImage *img = images[current];
  if(QFile::remove(img->file()) == false) {
    // TODO: Could not delete file.
  }
  if(current > 0) current--;
  doAbort();
}

void Grabber::doAbort()
{
  if(btn_del) delete btn_del;
  if(btn_abo) delete btn_abo;
  btn_del = btn_abo = NULL;
}

void Grabber::queryDelete()
{
  if(current == -1 || btn_del || btn_abo) return;

  int w = (int)(width() / 3.5);
  int h = (int)(height() / 2);

  QFont f = font();
  f.setPointSize(f.pointSize() * 3);
  f.setBold(true);

  btn_del = new QPushButton(this);
  btn_del->setText(tr("Delete"));
  btn_del->setFont(f);
  btn_del->resize(w, h);
  btn_del->move((int)(w * 0.5), (int)(h * 0.5));
  connect(btn_del, SIGNAL(clicked()), this, SLOT(doDelete()));

  btn_abo = new QPushButton(this);
  btn_abo->setText(tr("Abort"));
  btn_abo->setFont(f);
  btn_abo->resize(w, h);
  btn_abo->move((int)(width() - (w * 1.5)), (int)(h * 0.5));
  connect(btn_abo, SIGNAL(clicked()), this, SLOT(doAbort()));

  btn_del->show();
  btn_del->setDefault(true);
  btn_abo->show();
}


void Grabber::keyPressEvent(QKeyEvent *event)
{
  if(event->key() == Qt::Key_Left) {
    current--;
    if(current < 0) current = 0;
    repaint();
  }

  if(event->key() == Qt::Key_Right) {
    current++;
    if(current > images.size() - 1) current = images.size() - 1;
    repaint();
  }

  if(event->key() == Qt::Key_Escape) {
    if(btn_del && btn_abo) doAbort();
    else reject();
  }

  if(event->key() == Qt::Key_Return) {
    if(btn_del && btn_abo) doDelete();
    else accept();
  }

  if(event->key() == Qt::Key_Delete) {
    queryDelete();
  }
}
