/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set et sw=2 ts=2: */
/***************************************************************************
 *            image.cc
 *
 *  Wed Oct  6 13:54:25 CEST 2010
 *  Copyright 2010 Bent Bisballe Nyeng
 *  deva@aasimon.org
 ****************************************************************************/

/*
 *  This file is part of PC-Show.
 *
 *  PC-Show is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  PC-Show is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with PC-Show; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */
#include "image.h"

#include <QFile>
#include <QDir>

#include "imageloader.h"

Image::Image(const QString &p, const QString &f)
{
  path = p;
  file = f;
  image.load(":placeholder.png");

  imageloader->scheduleImage(this);
}

Image::~Image()
{
}

QImage Image::get(int w, int h)
{
  QImage i;
  mutex.lock();
  i = image.scaled(w, h);
  mutex.unlock();
  return i;
}

void Image::load()
{
  mutex.lock();
  if(!image.load(path + "/" + file)) {
    image.load(":error.png");
  }

  if(image.width() > 800 || image.width() > 600) {
    image = image.scaled(800, 600,
                         Qt::KeepAspectRatio,
                         Qt::SmoothTransformation);

  }
 
  mutex.unlock();
  emit loaded();
}

QString Image::filename()
{
  return path + QDir::separator() + file;
}

void Image::remove()
{
  mutex.lock();

  emit unloading();
  QFile::remove(filename());
  image = QImage();

  mutex.unlock();

}

void Image::rename(const QString &name)
{
  mutex.lock();

  if(name != file) {
    QFile::rename(filename(),  path + QDir::separator() + name);
    file = name;
    emit nameChanged();
  }

  mutex.unlock();
}

void Image::copy(const QString &dst)
{
  QFile::copy(filename(), dst + QDir::separator() + file);
}

void Image::move(const QString &dst)
{
  mutex.lock();

  if(dst != path) {
    QFile::rename(filename(),  dst + QDir::separator() + file);
    emit unloading();
  }

  mutex.unlock();
}

void Image::unload()
{
}

void Image::stopLoading()
{
}
