/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set et sw=2 ts=2: */
/***************************************************************************
 *            mainwindow.cc
 *
 *  Mon Aug 23 13:59:16 CEST 2010
 *  Copyright 2010 Bent Bisballe Nyeng
 *  deva@aasimon.org
 ****************************************************************************/

/*
 *  This file is part of PC-Show.
 *
 *  PC-Show is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  PC-Show is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with PC-Show; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */
#include "mainwindow.h"

#include <QMenuBar>
#include <QMenu>
#include <QAction>
#include <QKeySequence>
#include <QSettings>
#include <QCloseEvent>

#include <QFile>
#include <QFileDialog>

#include <QSettings>

#include <QStatusBar>

#include <QUrl>
#include <QDesktopServices>

#include <QDate>

#include "externalapp.h"
#include "optionswindow.h"

#include "print.h"

#include "fileoperations.h"

#include "aboutwindow.h"

#include "grabber.h"

#include "image.h"

MainWindow::MainWindow(QString path)
{
  setWindowTitle("PC-Show v"VERSION);
  setWindowIcon(QIcon(":pc-show.ico"));

  //installEventFilter(this);

  QMenuBar *menu = new QMenuBar();

	QAction *a;

  QMenu *filemenu = new QMenu(tr("&Files"));
  a = filemenu->addAction(tr("Show &selected"), this, SLOT(showSelected()),
                          QKeySequence(Qt::Key_Return));
  a = filemenu->addAction(tr("&Print selected"), this, SLOT(printSelected()),
                          QKeySequence(Qt::Key_U));
	a = filemenu->addSeparator();
	a = filemenu->addAction(tr("Show &fullscreen"), this, SLOT(showFullscreen()),
                          QKeySequence(Qt::Key_Z));
	a = filemenu->addSeparator();

  fileopmenu = new QMenu(tr("File &Operations"));
  QSettings settings(path + "/pc-show.ini", QSettings::IniFormat);
  settings.beginGroup("File Actions");

  if(settings.value("delete", "false").toString() == "true") {
    a = fileopmenu->addAction(tr("&Delete file"), this,
                            SLOT(deleteSelected()),
                            QKeySequence(Qt::CTRL + Qt::Key_D));
  }

  if(settings.value("copy", "false").toString() == "true") {
    a = fileopmenu->addAction(tr("&Copy file"), this,
                            SLOT(copySelected()),
                            QKeySequence(Qt::CTRL + Qt::Key_K));
  }

  if(settings.value("move", "false").toString() == "true") {
    a = fileopmenu->addAction(tr("&Move file"), this,
                            SLOT(moveSelected()),
                            QKeySequence(Qt::CTRL + Qt::Key_F));
  }
  
  if(settings.value("saveas", "false").toString() == "true") {
    a = fileopmenu->addAction(tr("&Save file as"), this,
                            SLOT(saveSelectedAs()),
                            QKeySequence(Qt::CTRL + Qt::Key_G));
  }
  
  if(settings.value("rename", "false").toString() == "true") {
    a = fileopmenu->addAction(tr("&Rename file"), this,
                            SLOT(renameSelected()),
                            QKeySequence(Qt::CTRL + Qt::Key_O));
  }
  
  settings.endGroup();

  /* // This menu only confuses the user...
  filemenu->addMenu(fileopmenu);
	a = filemenu->addSeparator();
  */

	a = filemenu->addAction(tr("E&xit"), this, SLOT(close()),
                          QKeySequence(Qt::ALT + Qt::Key_X));

  QMenu *toolsmenu = new QMenu(tr("&Tools"));
  a = toolsmenu->addAction(tr("&Options"), this, SLOT(showOptions()),
                          QKeySequence(Qt::CTRL + Qt::Key_O));
	a = toolsmenu->addSeparator();

  a = toolsmenu->addAction(tr("Activate &grabber"), this,
                            SLOT(showGrabber()),
                            QKeySequence(Qt::ALT + Qt::Key_G));

	a = toolsmenu->addSeparator();

  //    QSettings settings("pc-show.ini", QSettings::IniFormat);
  QStringList groups = settings.childGroups();
  QStringList::iterator i = groups.begin();
  while(i != groups.end()) {
    if(i->left(8) == "External") {
      settings.beginGroup(*i);
      
      QString bin = settings.value("bin", "").toString();
      if(bin != "") {
        QString name = settings.value("name", bin).toString();
        QString keys = settings.value("keys", "").toString();
        
        ExternalApp *app = new ExternalApp(bin);
        a = toolsmenu->addAction(name, app, SLOT(run()), keys);
      }
      
      settings.endGroup();
    }
    i++;
  }
  //}

  QMenu *helpmenu = new QMenu(tr("&Help"));
	a = helpmenu->addAction(tr("&Help"), this, SLOT(showHelp()));
	a = helpmenu->addAction(tr("&Online Help"), this, SLOT(showOnlineHelp()));
  a = helpmenu->addSeparator();
  a = helpmenu->addAction(tr("&Project website"), this, SLOT(showWebsite()));
  a = helpmenu->addAction(tr("&About PC-Show"), this, SLOT(showAbout()));
	
  menu->addMenu(filemenu);
  menu->addMenu(toolsmenu);
  menu->addMenu(helpmenu);
  setMenuBar(menu);

  pathwidget = new QLabel();
  statusBar()->addWidget(pathwidget);

  namewidget = new QLabel();
  statusBar()->addWidget(namewidget);

  filewidget = new FileWidget();
  connect(filewidget, SIGNAL(changeName(QString)),
          this, SLOT(changeName(QString)));
  connect(filewidget, SIGNAL(changePath(QString)),
          this, SLOT(changePath(QString)));
  addDockWidget(Qt::LeftDockWidgetArea, filewidget);

  {
    QSettings settings("Aasimon.org", "PC-Show");
    settings.beginGroup("Options");
    filewidget->setWatchFile(settings.value("watchfile",
                                            "c:\\x\\pcshow.tmp").toString());
    settings.endGroup();
  }

  viewer = new ViewerWidget(fileopmenu);
  setCentralWidget(viewer);

  connect(filewidget, SIGNAL(dblClicked(Image *)),
          viewer, SLOT(addImage(Image *)));

  connect(filewidget, SIGNAL(changePath(const QString &)),
          viewer, SLOT(clear()));

  {
    QSettings settings("Aasimon.org", "PC-Show");
    settings.beginGroup("MainWindow");
    resize(settings.value("size", QSize(800, 600)).toSize());
    move(settings.value("pos", QPoint(0, 0)).toPoint());
    settings.endGroup();
  }
}

void MainWindow::closeEvent(QCloseEvent *event)
{
  QSettings settings("Aasimon.org", "PC-Show");
  
  settings.beginGroup("MainWindow");
  settings.setValue("size", size());
  settings.setValue("pos", pos());
  settings.endGroup();

  event->accept();
}

bool MainWindow::eventFilter(QObject *, QEvent *)
{/*
  if(event->type() == QEvent::WindowActivate) {
    printf("Active\n");
    return true;
  }

  if(event->type() == QEvent::WindowDeactivate) {
    printf("Deactive\n");
    return true;
  }
 */
  return false;
}


void MainWindow::showSelected()
{
  ImageList sel = filewidget->selection();
  ImageList::iterator i = sel.begin();
  while(i != sel.end()) {
    viewer->addImage(*i);
    i++;
  }
}

void MainWindow::printSelected()
{
  ImageList sel = viewer->images();
  printSelection(sel);
}

void MainWindow::deleteSelected()
{
  if(viewer->hasSelection()) {
    ::deleteSelected(viewer->selected());
  }
}

void MainWindow::copySelected()
{
  if(viewer->hasSelection()) {
    ::copySelected(viewer->selected());
  }
}

void MainWindow::moveSelected()
{
  if(viewer->hasSelection()) {
    ::moveSelected(viewer->selected());
  }
}

void MainWindow::saveSelectedAs()
{
  if(viewer->hasSelection()) {
    ::saveSelectedAs(viewer->selected());
  }
}

void MainWindow::renameSelected()
{
  if(viewer->hasSelection()) {
    ::renameSelected(viewer->selected());
  }
}


void MainWindow::showFullscreen()
{
  if(viewer->hasSelection()) {
    viewer->selected()->showFullscreen();
  }
}

void MainWindow::showOptions()
{
  OptionsWindow opt;
  opt.exec();

  QSettings settings("Aasimon.org", "PC-Show");
  settings.beginGroup("Options");
  filewidget->setWatchFile(settings.value("watchfile",
                                          "c:\\x\\pcshow.tmp").toString());
  settings.endGroup();
}

void MainWindow::showHelp()
{
  QString helppage = QDir::tempPath() + "/help.html";
  QFile::copy(":help.html", helppage);
  QDesktopServices::openUrl(QUrl("file://" + helppage));
}

void MainWindow::showOnlineHelp()
{
  QDesktopServices::openUrl(QUrl("http://www.aasimon.org/pc-show/?page=help"));
}

void MainWindow::showWebsite()
{
  QDesktopServices::openUrl(QUrl("http://www.aasimon.org/pc-show"));
}

void MainWindow::showAbout()
{
  AboutWindow about;
  about.exec();
}

void MainWindow::changeName(QString name)
{
  namewidget->setText(name);
}

void MainWindow::changePath(QString path)
{
  this->path = path;
  pathwidget->setText(path);
}

void MainWindow::showGrabber()
{
  QSettings settings("Aasimon.org", "PC-Show");
  settings.beginGroup("Options");
  QString grabberpath = settings.value("grabberpath",
                                       tr("c:\\eye-fi")).toString();
  settings.endGroup();

  QDir d(grabberpath);
  QStringList sl = d.entryList(QDir::Files);

  Grabber g(grabberpath);
  if(g.exec() == QDialog::Accepted) {
    // Move all files from grabber path to current patient folder.
    QStringList::iterator i = sl.begin();
    while(i != sl.end()) {
      unsigned int cnt = 0;
      QFileInfo inf(*i);
      QString suffix = "." + inf.completeSuffix();
      QString date = QDate::currentDate().toString(Qt::ISODate);
      QString target;
      do {
        target = path + "/" + date + "-" + QString::number(cnt++) + suffix;
      } while(d.exists(target));

      if(d.rename(*i, target) == false) {
        // TODO: Display message: Could not move file... perms?
      }

      i++;
    }
  }

  QStringList::iterator i = sl.begin();
  while(i != sl.end()) {
    if(d.remove(*i) == false) {
      // TODO: Display message: Could not remove file... perms?
    }
    i++;
  }
}
