/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set et sw=2 ts=2: */
/***************************************************************************
 *            print.cc
 *
 *  Wed Aug 25 20:59:52 CEST 2010
 *  Copyright 2010 Bent Bisballe Nyeng
 *  deva@aasimon.org
 ****************************************************************************/

/*
 *  This file is part of PC-Show.
 *
 *  PC-Show is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  PC-Show is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with PC-Show; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */
#include "print.h"

#include <QObject>

#include <QPrinter>
#include <QPrintDialog>
#include <QPainter>
#include <QTransform>
#include <QFileInfo>
#include <QPicture>
#include <QApplication>
#include <QCursor>

static void printSubPage(QPainter &painter,
                         ImageList &sel,
                         int pw, int ph,
                         int offset = 0)
{
  // ph, pw: page height page width
  // ih, iw: image height, image width
  // th: text height
  // border: border width or height

  int border = ph / 40;
  int th = border;

  int iw = (pw / 2) - border;
  int ih = (ph / 2) - (border + th);

  int pos = 0;
  ImageList::iterator i = sel.begin();
  while(i != sel.end()) {
    Image *img = *i;
    QImage image = img->image.scaled(iw, ih,
                                     //Qt::IgnoreAspectRatio,
                                     Qt::KeepAspectRatio,
                                     //Qt::FastTransformation);
                                     Qt::SmoothTransformation);

    int wdiff = iw - image.width();

    int posx = (pw / 2) * (pos % 2)  + (border / 2);
    int posy = (ph / 2) * (pos / 2) + offset;

    painter.drawImage(posx + (wdiff / 2), posy, image);

    QFileInfo file((*i)->filename());
    painter.drawText(QRect(posx, posy + ih,
                           iw, th),
                     Qt::AlignCenter,
                     file.fileName());

    pos++;
    i++;
  }
}

static void printPage(QPainter &painter, ImageList &lst, int w, int h)
{
  if(lst.size() <= 4) {

    QTransform rot;
    rot.translate(w, 0);
    rot.rotate(90);
    painter.setWorldTransform(rot);

    printSubPage(painter, lst, h, w);

  } else {


    ImageList lst2;
    for(int p = 0; p < 4; p++) {
      lst2.push_back(lst.front());
      lst.pop_front();
    }

    printSubPage(painter, lst, w, h/2);
    printSubPage(painter, lst2, w, h/2, h/2);

  }
}

void printSelection(ImageList &sel)
{
  QPrinter printer(QPrinter::HighResolution);

  QPrintDialog *dialog = new QPrintDialog(&printer, NULL);
  dialog->setWindowTitle(QObject::tr("Print Document"));
  if(dialog->exec() != QDialog::Accepted) return;

  QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));

  QRect r = printer.pageRect();

  int w = r.width();
  int h = r.height();

  QPainter painter;
  painter.begin(&printer);

  ImageList::iterator i = sel.begin();

  while(i != sel.end()) {
    ImageList il;
    int num = 0;

    if(i != sel.begin()) printer.newPage();

    while(i != sel.end() && num < 8) {
      il.push_back(*i);
      i++;
      num++;
    }

    printPage(painter, il, w, h);
  }

  painter.end();

  QApplication::restoreOverrideCursor();
}

/*
void printSelection(ImageList &sel)
{
  QPrinter printer(QPrinter::HighResolution);

  QPrintDialog *dialog = new QPrintDialog(&printer, NULL);
  dialog->setWindowTitle(QObject::tr("Print Document"));
  if(dialog->exec() != QDialog::Accepted) return;

  QRect r = printer.pageRect();

  int w = (int)(r.width() / 2.0);
  int h = (int)(r.height() / 2.0);

  int border = (int)(w * 0.025);

  QPainter painter;
  painter.begin(&printer);

  int num = 0;

  int len = sel.size();
  ImageList::iterator i = sel.begin();
  while(i != sel.end()) {

    if(num > 3) {
      printer.newPage();
      num = 0;
    }

    QTransform rot;
    rot.rotate(90);
    QImage image = (*i)->image.transformed(rot);
    image = image.scaled((int)(w * 0.95), (int)(h * 0.95),
                         Qt::KeepAspectRatio, Qt::SmoothTransformation);
      
    int pos;
    switch(num) {
    case 0:
      pos = 1;
      break;
    case 1:
      pos = 3;
      break;
    case 2:
      pos = 0;
      break;
    case 3:
    default:
      pos = 2;
      break;
    }
    painter.drawImage(w * (pos % 2) + border,
                      h * (pos / 2) + border * 2,
                      image);
      
    QFileInfo file((*i)->filename());
    painter.drawText(QRect(w * (pos % 2),
                           h * (pos / 2),
                           w, border * 2),
                     Qt::AlignCenter,
                     file.fileName());

    len--;
    num++;
    i++;
  }

  painter.end();
}
*/
